% Usage V = cheb (xs, ns)
%
% V(i,j) = cheb(xs(i),ns(j))
%
% If xs is single precision, V will also be single precision
%
% $Id: cheb.m,v 1.1 2012/03/10 20:51:16 shiv Exp shiv $
%

function V = cheb (xs, ns)
  % check xs
  M = max(size(xs));
  % assume column xs
  if 1 == rows(xs)
    xs = xs';
  endif
  % assume row ns
  N = max(size(ns));
  if 1 == columns(ns)
    ns = ns';
  endif
  % Explicitly allow for single precision
  V = zeros(M, N, class(xs));
  %
  i = find(abs(xs) <= 1);
  if length(i) > 0 
    V(i, :) = cos(acos(xs(i)) * ns);
  endif
  %
  i = find(1 < xs);
  if length(i) > 0
    V(i, :) = cosh(acosh(xs(i)) * ns);
  endif
  %
  i = find(xs < -1);
  if length(i) > 0
    V(i, :) = cosh(acosh(-xs(i)) * ns) * diag ((-1) .^ ns);
  endif
  %
endfunction


% Local Variables: %
% mode: octave     %
% End:             %
