% Usage: V = cheb_deriv(xs, ns)
%
% V(i,j) = cheb'(xs(i), ns(j))
%
% Derivative of cheb(n,x)
% | = n sin(n acos(x)) / sqrt(1-x^2),             -1 < x < 1
% | = n sinh(n acosh(x))/ sqrt(x^2-1),             1 < x
% | = (-1)^{n+1} n sinh(n acosh(-x))/ sqrt(x^2-1), x < -1
% | = n^2                                          x = 1
% | = (-1)^{n+1}n^2, 0 < n,                        x = -1
% | = 0, n = 0
%
% $Id: cheb_deriv.m,v 1.1 2012/03/11 07:09:42 shiv Exp shiv $
%

function V = cheb_deriv (xs, ns)
  %
  M = max(size(xs));
  N = max(size(ns));
  if 1 == rows(xs)
    xs = xs';
  endif
  assert (columns(xs), 1);
  if 1 == columns(ns)
    ns = ns';
  endif
  assert (rows(ns), 1);
  V = zeros(M, N, class(xs));
  %
  x2 = xs .* xs;
  %
  i = find(x2 < 1);
  if length(i) > 0
    V(i,:) = diag(1./sqrt(1-x2(i))) * sin(acos(xs(i)) * ns) * diag(ns);
  endif
  %
  i = find(1 < xs);
  if length(i) > 0
    V(i,:) = diag(1./sqrt(x2(i)-1)) * sinh(acosh(xs(i)) * ns) * diag(ns);
  endif
  %
  i = find(xs < -1);
  if length(i) > 0
    V(i,:) = diag(1./sqrt(x2(i)-1)) * sinh(acosh(-xs(i)) * ns) * \
	diag(((-1) .^ (ns+1)) .* ns);
  endif
  %
  i = find(xs == 1);
  if length(i) > 0
    V(i,:) = ones(length(i),1) * (ns .* ns);
  endif
  %
  i = find(xs == -1);
  if length(i) > 0
    V(i,:) = ones(length(i),1) * (((-1) .^ (ns+1)) .* (ns .* ns));
  endif
  %
  j = find(ns == 0);
  if length(j) > 0
    V(:,j) = 0;
  endif
  %
endfunction

% Local Variables: %
% mode: octave     %
% End:             %
