# Usage [as, ord] = msn_interp2D (XY, xs, fs, s, N1, N2)
#
# XY --- 4x2, rows are vertices of convex polynomial in anti-clockwise order
# xs --- Sample points (in XY hopefully)
# fs --- Function at sample points
# s --- Sobolev weight
# N = [N1 N2] --- Order of 2D Chebyshev polynomials
# as --- Chebyshev coefficients
#
# $Id: msn_interp2D.m,v 1.1 2012/06/01 00:54:57 shiv Exp shiv $
#

function [as, ord] = msn_interp2D (XY, xs, fs, s, N)
  N1 = N(1);
  N2 = N(2);
  zs = fromQuad (XY, xs);
  m = rows (zs);
  n = N1 * N2;
  ord = [(kron ([0 : N1-1]', ones(N2, 1))), \
	 (kron (ones(N1, 1), [0 : N2-1]'))];
  V = cheb2D (zs, ord);
  Dsi = diag ((1+ord(:,1)+ord(:,2)) .^ (-s));
  V = V * Dsi;
  if rows(fs) != m
    fs = fs';
  endif
  # [u, l, v] = rrulv (V);
  [u, l, v] = rrulv2 (V, fs);
  # v = vt';
  as = Dsi * (v * (l \ (u' * fs)));
endfunction

# Local Variables: #
# mode: octave     #
# End:             #
