# Usage fp = persp_deriv (A, xy)
#
# Compute Jacobian of persp map: f: (x, y) -> (p, q) where
#                                A*[xy; 1] = [p*z; q*z; z];
#
# fp = [f1_x, f1_y;
#       f2_x; f2_y];
#
# See Papers/MSNFEM/geom2Dcheb.tm
#
# $Id$
#

function f = persp_deriv (A, xy)
  #
  if columns(xy) == 1
    xy = xy'
  endif
  pqz = A * [xy'; 1];
  z = pqz(3);
  p = pqz(1);
  q = pqz(2);
  #
  f = [A(1,1)*z - p*A(3,1), A(1,2)*z - p*A(3,2);
       A(2,1)*z - q*A(3,1), A(2,2)*z - q*A(3,2)] / z^2;
  #
endfunction

# Local Variables: #
# mode: octave     #
# End:             #
