# Usage A = quad2quad(XY, PQ)
#
# Compute persective transform that maps quadrilateral XY to PQ
# XY(i,:) = (xi, yi) --- i-th vertex of XY
# PQ(i,:) = (pq, qi) --- i-th vertex of PQ
# A --- 3 x 3 perspective transform
#
# if A*[x; y; 1] = [p*r; q*r; r]
# A^-1 is the inverse transform
#
# See Papers/MSNFEM/geom2Dcheb.tm
#
# $Id$
#

function A = quad2quad (XY, PQ)
  #
  x1 = XY(1,1); y1 = XY(1,2);
  x2 = XY(2,1); y2 = XY(2,2);
  x3 = XY(3,1); y3 = XY(3,2);
  x4 = XY(4,1); y4 = XY(4,2);
  #
  p1 = PQ(1,1); q1 = PQ(1,2);
  p2 = PQ(2,1); q2 = PQ(2,2);
  p3 = PQ(3,1); q3 = PQ(3,2);
  p4 = PQ(4,1); q4 = PQ(4,2);
  #
  B = [x1 0  -p1*x1 y1 0  -p1*y1 1 0 -p1;
       0  x1 -q1*x1 0  y1 -q1*y1 0 1 -q1;
       x2 0  -p2*x2 y2 0  -p2*y2 1 0 -p2;
       0  x2 -q2*x2 0  y2 -q2*y2 0 1 -q2;
       x3 0  -p3*x3 y3 0  -p3*y3 1 0 -p3;
       0  x3 -q3*x3 0  y3 -q3*y3 0 1 -q3;
       x4 0  -p4*x4 y4 0  -p4*y4 1 0 -p4;
       0  x4 -q4*x4 0  y4 -q4*y4 0 1 -q4];
  #
  [qt, rt] = qr(B'); # full QR factorization
  A = qt(:,9);
  A = reshape(A,3,3);
  #
endfunction

# Local Variables: #
# mode: octave     #
# End:             #
