## 
## Usage rGeom = refineGridGlobal (Geom, level)
## 
## Geom -- input geometry to be refined
## level -- number of times to refine (level == 0 => no refinement)
## rGeom -- refined geometry
## 
## $Id$
## 

## 
## Note: CG of convex quadrilateral is also the following intersection point:
## Join the midpoint of opposite sides, and take their intersection.
## The CG to mid-point lines give 4 new sub convex quadrilaterals.
## 
## For each line place a new point between adjacent points on the line
## For each quadrilateral add CG as a new point
## For each quad, add a line through CG and 2 horizontal size mid-points
## For each quad, add a line through CG and 2 vertical size mid-points
## For each quad add 4 new edges
## For each quad replace with 4 new sub-quads
## 

## 
## I count everything first so that I know how to implement in ATS (GC free)
## 

function rGeom = refineGridGlobal (Geom, level)
  ## 
  pGeom = Geom;
  ## 
  for lev = 1 : level
    ##
    ## Old counts
    pNl = length (pGeom.Lines);	# no. of lines
    pNq = length (pGeom.Quads);	# no. of quads
    pNe = rows (pGeom.Edges);	# no. of edges
    pNp = rows (pGeom.Pts);	# no. of points
    ## 
    ## New counts
    cNq = 4 * pNq;		# 4 news quads per old quad
    cNl = pNl + 2 * pNq;	# 2 new lines per old quad
    cNp = pNq;			# 1 new point per old quad plus ...
    for i = 1 : Nl
      pnl = length (pGeom.Lines{i});
      cNp = cNp + 2 * pnl - 1;
    endfor
    cNe = 2 * pNe + 4 * pNq;	# 2 per old edge + 4 per old quad
    ## 
    ## Allocate new Geometry
    cGeom.Pts = nan * ones (cNp, 2); # filled with Nan's
    cGeom.Lines = cell (cNl, 1);
    cGeom.Edges = zeros (cNe, 3);
    cGeom.Quads = cell (cNq, 1);
    ## 
    ## Copy over old points
    cGeom.Pts(1:pNp,:) = pGeom.Pts;
    cpt = pNp + 1; 		# where the next point goes
    ## 
    ## Midpoint of each pair of adjacent points on every line
    for l = 1 : pNl
      nl = length (pGeom.Lines{l});
      cGeom.Lines{i} = zeros (2*nl-1, 1);
      for j = 1 : nl-1
	prev = pGeom.Lines{l}(j);
	nxt = pGeom.Lines{l}(j+1);
	mid_pt = (pGeom.Pts(prev,:) + pGeom.Pts(nxt,:)) / 2;
	cGeom.Pts(cpt,:) = mid_pt; # append to global set of points
	cGeom.Lines{l}(2*j-1) = prev;
	cGeom.Lines{l}(2*j) = cpt;
	cGeom.Lines{l}(2*j+1) = nxt;
	cpt = cpt + 1;
      endfor
    endfor
    ##
    cln = pNl + 1;		# where the next line goes
    ## CG of each quad
    quadCGs = nan * ones (pNq, 2); # index of each cg and new line
    for q = 1 : pNq
      vs = getVerticesOfQuad (pGeom, q);
      pts = pGeom.Pts(vs,:);
      cg = [sum(pts(:,1)) sum(pts(:,2))] / 4;
      cGeom.Pts(cpt) = cg;
      ## Add 2 new lines
      cGeom.Lines{cln} = [vs(1) cpt vs(3)];
      cGeom.Lines{cln+1} = [vs(2) cpt vs(4)];
      ## Update indices
      quadCGs(q,:) = [cpt cln];
      cpt = cpt + 1;
      cln = cln + 2;
    endfor
    ## 
    assert (cpt, cNp+1);	# sanity check
    ##
    ## Create 2 new edges per old edge
    for i = 1 : pNe
      l = pGeom.Edges(i,1);
      st = pGeom.Edges(i,2);
      en = pGeom.Edges(i,3);
      midPt = (pGeom.Pts(st,:) + pGeom.Pts(en,:)) / 2;
      ## Find mid-point from discrete set by scanning
      pts = cGeom.Lines{i};	# from current Geometry
      npts = length (pts);
      midInd = -1;
      dist = Inf;
      for j = 1 : npts
	dj = norm (midPt - cGeom.Pts(pts(j), :));
	if dj < dist
	  dist = dj;
	  midInd = j;
	endif
      endfor
      assert (dist < Inf);
      ## 
      cGeom.Edges(2*i-1,:) = [l st midInd];
      cGeom.Edges(2*i,:) = [l midInd en];
    endfor
    ##
    ## Create 4 new edges per old quad
    ced = 2 * pNe + 1;		# where next new edge goes
    for q = 1 : pNq
      vs = getVerticesOfQuad (pGeom, q);
      cg = quadCGs (q, 1);
      ln1 = quadCGs (q, 2);
      ln2 = ln1 + 1;
      ## SW sub-quad
    endfor
    ## 
    ## Create 4 new quads per old quad
    ## 
    pGeom = cGeom;
  endfor			# for lev = 1 : level
  ## 
  rGeom = cGeom;
  ##   
endfunction


# Local Variables: #
# mode: octave     #
# End:             #
