% Usage: [u,l,v] = rrulv(A, B)
%
% u * l * v' = A with A being a fat matrix
% l should be in rank-revealed lower-triangle form.
% If A is badly column-scaled, l should reveal even tiny singular
% values accurately.
% For well-conditioned A, l should be numerically diagonal.
% v is a skinny orthonormal matrix.
% u is a orthogonal matrix.
% 
% If B is provided, we also return B*full(v), where full(v) refers to the
% full orthogonal matrix v. Note that B*full(v) is returned in place of v.
%
% $Id: rrulv.m,v 1.4 2012/05/31 20:59:57 shiv Exp shiv $
%

function [u, l, v] = rrulv (A, B)
  [m, n] = size (A);
  assert(m <= n);
  %
  colnrms = zeros (n,1);
  for j=1:n
    colnrms(j) = norm (A(:,j),Inf);
  endfor
  [cn, piJ] = sort (colnrms, "descend");
  ApiJ = A(:,piJ);
  %
  if (nargin > 1)
    BpiJ = B(:,piJ);
  else
    BpiJ = [];
  endif
  %
  [u, _, _] = svd (ApiJ,1);
  %%%%
  A2 = u' * ApiJ;
  %%%%%%%%%%
  # m2 = floor(m/2);
  # [u2, _ , _] = svd(A2(m2+1:end,:),1);
  # A2(m2+1:end,:) = u2' * A2(m2+1:end,:);
  # u(:, m2+1:end) = u(:, m2+1:end) * u2;
  %%%%%%%%%%
  # epsilon = mach_eps(cast(1,class(A)));
  # for j=1:n
  #   for i=1:m
  #     if abs(A2(i,j)) <= epsilon * cn(j)
  # 	A2(i,j) = 0;
  #     endif
  #   endfor
  # endfor
  %%%%
  if (nargin > 1)
    [fullv, lt, P] = qr (A2');
    lt = lt(1:m,1:m);
    Bv = BpiJ * fullv;
    v = [];
  else
    [v, lt, P] = qr (A2', 0);
    P = (eye(m,m))(P,:);
  endif
  u = u * P;
  %
  if (nargin == 1)
    invpiJ = zeros (n,1);
    for j=1:n
      invpiJ(piJ(j)) = j;
    endfor
    v = v(invpiJ,:);
  else
    v = Bv;
  endif
  l = lt';
  %
endfunction

% Local Variables: %
% mode: octave     %
% End:             %
