% Usage: [u, l, v] = rrulv2 (A, C)
##       [u, l, Bv] = rrulv2 (A, C, B)
%
% u * l * v' = A with A being a fat matrix
% l should be in rank-revealed lower-triangle form.
% If A is badly column-scaled, l should reveal even tiny singular
% values accurately.
% For well-conditioned A, l should be numerically diagonal.
% v is a skinny orthonormal matrix.
% u is a orthogonal matrix.
% 
% If B is provided, we also return B*full(v), where full(v) refers to the
% full orthogonal matrix v. Note that B*full(v) is returned in place of v.
## 
## C is provided to ensure that u' * C is well-graded too.
## Normally you would pick C to be the RHS of the linear system for which you
## are seeking a minimum norm solution.
## 
## Compare with rrulv.m for a version that does not require C
%
% $Id$
%

function [u, l, v] = rrulv2 (A, C, B)
  [m, n] = size (A);
  assert(m <= n);
  %
  colnrms = norm (A, Inf, "columns");
  [cn, piJ] = sort (colnrms, "descend");
  ApiJ = A(:,piJ);
  %
  if (nargin == 3)
    BpiJ = B(:,piJ);
  else
    BpiJ = [];
  endif
  %
  [u, _, _] = svd ([C ApiJ],1);
  %
  A2 = u' * ApiJ;
  %
  if (nargin == 3)
    [fullv, lt, P] = qr (A2');
    lt = lt(1:m,1:m);
    Bv = BpiJ * fullv;
    v = [];
  else
    [v, lt, P] = qr (A2', 0);
    P = (eye(m,m))(P,:);
  endif
  u = u * P;
  %
  if (nargin == 2)
    invpiJ = zeros (n,1);
    for j=1:n
      invpiJ(piJ(j)) = j;
    endfor
    v = v(invpiJ,:);
  else
    v = Bv;
  endif
  l = lt';
  %
endfunction

% Local Variables: %
% mode: octave     %
% End:             %
