## Test 2 of msnODE.m
## [2 -1; -1 2]u'(x) + lambda [0 1; 1 0] u(x) = f(x)
## u1(0), u2(1) 
## q = 2

## $Id$

one = single(1);			# single(1) for single precision
realType = class(one);		# Single or double

s = 12*one;			# Sobolev parameter
Np = 3;			# Number of patches
ys = cast([0:1/(Np-1):1], realType);		# The patches
assert(length(ys), Np);
ns = 100 * ones(1,Np-1); # Points per patch
M1 = 2.0*one;		# Order of MSN poly / mesh_norm

q = 2;

## A = [2 -1; -1 2]
function A = coeff1 (x)
  A = cast([2 -1; -1 2], class(x));
endfunction

## B = [0 1; 1 0]
function B = coeff0 (x)
  lambda = cast(1.0, class(x));
  B = lambda * cast([0 1; 1 0], class(x));
endfunction

## u(x) = kron(exp(x*l),[2;3])
## l and x can be arrays
function u = soln (x, l)
  if columns(x) > 1
    my_x = x';
  else
    my_x = x;
  endif
  if rows(l) > 1
    my_l = l';
  else
    my_l = l;
  endif
  u = kron(exp (my_x * my_l), [2; 3]);
endfunction

## u'(x) = kron(exp(x*l)*l,[2;3])
function up = solnp (x, l)
  if columns(x) > 1
    my_x = x';
  else
    my_x = x;
  endif
  if rows(l) > 1
    my_l = l';
  else
    my_l = l;
  endif
  up = kron(exp (my_x*my_l) * diag(my_l), [2; 3]);
endfunction

CL = cast([1 0], realType);
CR = cast([0 1], realType);

global l
l = cast([-2.0, -1.0, 1.0, 2.0], realType);
nrhs = columns(l);
gL = CL*soln(ys(1), l);
gR = CR*soln(ys(Np), l);

function f = rhs(x)
  global l;
  m = length(x);
  k = length(l);
  f = zeros(m*2, k, class(x));
  for i=1:m
    f((i-1)*2+1:i*2,:) = coeff1(x(i))*solnp(x(i), l) + coeff0(x)*soln(x(i), l);
  endfor
endfunction

us1 = msnODE (q, @coeff1, @coeff0, @rhs, CL, CR, gL, gR, s, M1, ys, ns);
us2 = soln(ys(2:Np-1), l);
err = norm(us1-us2,Inf) / norm(us2,Inf)

# Local Variables: #
# mode: octave     #
# End:             #
