## Test 3 of msnODE.m
## Bessel's equation
## x^2 u''(x) + x u'(x) + (x^2 - alpha^2) u(x) = 0
## alpha >= 0
## u(0) = besselj(alpha,0)
## u(T) = besselj(alpha,T)
## 
## [x^2 0; 0 1][u'; u] + [x (x^2-alpha^2); -1 0] [u'; u] = [0; 0]
## u2(0), u2(T)
## q = 2

## $Id: test_msnode_1.m,v 1.1 2012/03/14 04:23:01 shiv Exp shiv $

one = 1;			# single(1) for single precision
realType = class(one);		# Single or double

s = 12*one;			# Sobolev parameter
Np = 10;			# Number of patches
T = 1;
ys = cast(T*[0:1/(Np-1):1], realType);		# The patches
ys = ys .* ys;
#ys = [0 to_interval(0, T, cheb_zeros(Np-2)) T];
assert(length(ys), Np);
ns = 70 * ones(1,Np-1); # Points per patch
M1 = 2.0*one;		# Order of MSN poly / mesh_norm

q = 2;

## A = [x^2 0; 0 1]
function A = coeff1 (x)
  A = cast([x^2 0; 0 1], class(x));
endfunction

global alpha
alpha = cast(0.75, realType);
## B = [x (x^2-alpha); -1 0]
function B = coeff0 (x)
  global alpha
  B = cast([x (x^2-alpha^2); -1 0], class(x));
endfunction

## u(x) = [besselj'(alpha,x); besselj(alpha,x)]
## besselj'(alpha,x) = 0.5(besselj(alpha-1,x)-besselj(alpha+1,x))
function u = soln (x)
  global alpha
  m = length(x);
  u = zeros(2*m,1,class(x));
  for i=1:m
    u((i-1)*2+1:i*2,1) = [(besselj(alpha-1,x(i))-besselj(alpha+1,x(i)))/2;
			besselj(alpha, x(i))];
  endfor
endfunction

nrhs = 1;

CL = cast([0 1], realType);
# gL = CL*soln(ys(1)); infty!
gL = [besselj(alpha, ys(1))]

CR = cast([0 1], realType);
gR = CR*soln(ys(Np))

function f = rhs(x)
  m = length(x);
  f = zeros(m*2, 1, class(x));
endfunction

us1 = msnODE (q, @coeff1, @coeff0, @rhs, CL, CR, gL, gR, s, M1, ys, ns);
us1 = reshape(us1, 2, Np-2);
us1 = us1';
us2 = soln(ys(2:Np-1));
us2 = reshape(us2, 2, Np-2);
us2 = us2';
err = norm(us1(:,2)-us2(:,2),Inf) / norm(us2(:,2),Inf)

# Local Variables: #
# mode: octave     #
# End:             #
