## Test 15 of msnPDE2.m
##
## Dis-continuous div-curl
## 
## $Id$
##

##################################
## Two rectangles side by side. ##
##################################
##
##    <--E3--><--E4--->
##    <-------L2------>	
## 	    	       	
## ^  4-------5-------6 ^
## |  |	    ^ |       |	|
## |  |	    | |	      |	|
## L3 |    L4 |       |	L5     	    
## |  |     | |	      |	|   
## |  |	    v |	      |	|
## v  1-------2-------3	v
##    		       
##    <------L1------->
##    <--E1--><--E2--->
##

#		
Pts = [0 0;			# 1
       1 0;			# 2
       2 0;			# 3
       0 1;			# 4
       1 1;			# 5
       2 1];			# 6
#		
Lines = {[1 2 3];		# 1
	 [4 5 6];		# 2
	 [1 4];			# 3
	 [2 5];			# 4
	 [3 6]};		# 5
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 2 4 5;			# 3
	 2 5 6;			# 4
	 3 1 4;			# 5
	 4 2 5;			# 6
	 5 3 6];		# 7
#
BEdges = [1 2 3 4 5 7];
#
IEdges = [6];
#
# Anti-clockwise
Quads = {[1 6 -3 -5];		# 1
	 [2 7 -4 -6]};		# 2
#
## Create a grid on E6
ne = 13;
igrid = {toEdge(Pts(2,:), Pts(5,:), cheb_zeros(ne))};
#
nse = ne * ones(7,1);		# points on edges
##
ss = 10.0 * ones(2,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = [nx ny;			# No. of points for each Quad
      nx ny];
#
M1 = 2.5;			# MSN amp factor
## 
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
##### Discontinuous DIV-CURL ################
#############################################

p = 2;
q = 2;
nrhs = 1;

function A1 = coeff1x (j, xy)
  if j==1
    A1 = [1 0; 0 1];
  elseif j==2
    A1 = [2 0; 0 1];
  else
    error ("Unknown quad");
  endif
  A1 = cast (A1, class(xy));
endfunction

function A2 = coeff1y (j, xy)
  if j==1
    A2 = [0 1; -1 0];
  elseif j==2
    A2 = [0 2; -1 0];
  else
    error ("Unknown quad");
  endif
  A2 = cast (A2, class (xy));
endfunction

function B = coeff0 (j, x)
  B = zeros (2, 2, class (x));
endfunction

function u = soln (j, xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       x^2 - 2*y^2 + x*y - x + 1];
  if j==1
    u(1) = 2*u(1);
  endif
  # u = [1+x-y+x*y+x^3; \
  #      x+y-x^2+y^2-5*y*x^2];
  u = cast (u, class (xy));
endfunction

# [u1_x u1_y; u2_x u2_y]
function up = solnp (j, xy)
  x = xy(1);
  y = xy(2); 
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	2*x+y-1, -4*y+x];
  if j==1
    up(1,:) = 2*up(1,:);
  endif
  # up = [1+y+3*x^2, -1+x; \
  # 	1-2*x-10*y*x, 1+2*y-5*x^2];
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  u = soln (j, xy);
  gu = solnp (j, xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  f = cast (f, class (xy));
endfunction

function C = boundcoeff(j, xy)
  if     j==1
    C = [0 -1];			# normal
  elseif j==2
    C = [0 -1];			# normal
  elseif j==3
    C = [0 1];			# normal
  elseif j==4
    C = [0 1];			# normal
  elseif j==5
    C = [0 -1];			# tangential
  elseif j==6
    error("Interface");
  elseif j==7
    C = [1 0];			# normal
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (j, xy);
  C = boundcoeff (j, xy);
  g = C * u;
endfunction

function jm = Jm (p, j, xy)
  if j==6
    if p==1
      jm = [1 0; 0 1];
    elseif p==2
      jm = [2 0; 0 1];
    else
      error ("Unknown quad");
    endif
  else
    error ("Should not be called");
  endif
  jm = cast (jm, class(xy));
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE2 (Geom, igrid, q, p, nrhs, \
		      @coeff1x, @coeff1y, @coeff0, @rhs, \
		      @boundcoeff, @boundrhs, @Jm, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  edge = zzs(i,3);
  quad = -1;
  for j = 1 : length (Quads)
    if any (abs(Quads{j}) == edge)
      quad = j;
      break;
    endif
  endfor
  assert (quad > 0);
  us2(:,i) = Jm (quad, edge, zzs(i,1:2)) * soln (quad, zzs(i,1:2));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
