## Test 15_3 of msnPDE2.m
##
## Dis-continuous div-curl on L-shaped region
## 
## $Id$
##

######################################
## Two quadrilaterals side by side. ##
######################################
## 			    
## 			    
## 			    
## 			    <--L3-->
##	       	       	       	    
##                     	  ^ 5------6 ^
##                     	  | |	   | |
## 	    	       	 L6 |  	   | | 
##                        | | 	   | | 
##        <------L2-------|>| 	   | | 
##                        v |  	   | | 
##      ^ 3-----------------4  	   | | 
##      | |                A \ 	   | | 	  
##      | |                 \ \	   | L7
##      | |                 L5 \   | | 
##     L4 |	       	      \	\  | |
##      | |            	       \ \ | |
##      | |            	        J \| |
##     	v 1------------------------2 v
## 	   			    
## 	  <-----------L1----------->
##     	  
## 
## 

#		
global Pts;
Pts = [0 0;			# 1
       2 0;			# 2
       0 1;			# 3
       1 1;			# 4
       1 2;			# 5
       2 2];			# 6
#	  	
Lines = {[1 2];		# 1
	 [3 4];		# 2
	 [5 6];		# 3
	 [1 3];		# 4
	 [2 4];		# 5
	 [4 5];		# 6
	 [2 6]};	# 7
#
global Edges;
Edges = [1 1 2;			# 1
	 2 3 4;			# 2
	 3 5 6;			# 3
	 4 1 3;			# 4
	 5 2 4;			# 5
	 6 4 5;			# 6
	 7 2 6];		# 7
#
BEdges = [1 2 3 4 6 7];
#
IEdges = [5];
#
# Anti-clockwise
global Quads;
Quads = {[1 5 -2 -4];		# 1
	 [7 -3 -6 -5]};		# 2
#
## Create a grid on E5
ne = 13;
igrid = {toEdge(Pts(2,:), Pts(4,:), cheb_zeros(ne))};
#
nse = ne * ones(7,1);		# points on edges
##
ss = 10.0 * ones(2,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = [nx ny;			# No. of points for each Quad
      nx ny];
#
M1 = 2.5;			# MSN amp factor
## 
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
##### Discontinuous DIV-CURL ################
#############################################

p = 2;
q = 2;
nrhs = 1;

## coefficient inside divergence term
function A = divCoeff (j, xy)
  if j==1
    A = [1 0; 0 1];
  elseif j==2
    A = [2 0; 0 2];
  else
    error ("Unknown quad");
  endif
  A = cast (A, class(xy));
endfunction

## [A11_x A12_x; A21_x A22_x], [A11_y A12_y; A21_y A22_y]
function [Apx, Apy] = divCoeffJ (j, xy)
  Apx = [0 0; 0 0];
  Apy = [0 0; 0 0];
  Apx = cast (Apx, class(xy));
  Apy = cast (Apy, class(xy));
endfunction

function A1 = coeff1x (j, xy)
  A = divCoeff (j, xy);
  A1 = [A(1,1) A(1,2); 0 1];
  A1 = cast (A1, class(xy));
endfunction

function A2 = coeff1y (j, xy)
  A = divCoeff (j, xy);
  A2 = [A(2,1) A(2,2); -1 0];
  A2 = cast (A2, class (xy));
endfunction

function B = coeff0 (j, x)
  [Apx, Apy] = divCoeffJ (j, x);
  B = [(Apx(1,1)+Apy(2,1)), (Apx(1,2)+Apy(2,2)); 0 0];
  B = cast (B, class (x));
endfunction

function tngnt = tangent (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  tngnt = vt2 - vt1;
  tngnt = tngnt / norm(tngnt);
endfunction

## Rotate by 90
function nrm = rot90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [t(2), -t(1)];
  nrm = nrm / norm(nrm);
endfunction

## Rotate by -90
function nrm = rot_90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [-t(2), t(1)];
  nrm = nrm / norm(nrm);
endfunction

function jm = Jm (p, j, xy)
  A = divCoeff (p, xy);
  n = rot90 (5);
  t = tangent (5);
  jm = [n*A; t];
  jm = cast (jm, class(xy));
endfunction

function u = soln (j, xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       x^2 - 2*y^2 + x*y - x + 1];
  jm = Jm (j, 5, xy);
  u = jm \ u;
  # u = [1+x-y+x*y+x^3; \
  #      x+y-x^2+y^2-5*y*x^2];
  u = cast (u, class (xy));
endfunction

# [u1_x u1_y; u2_x u2_y]
function up = solnp (j, xy)
  x = xy(1);
  y = xy(2); 
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	2*x+y-1, -4*y+x];
  jm = Jm (j, 5, xy);
  up = jm \ up;			# assuming jm is constant
  # up = [1+y+3*x^2, -1+x; \
  # 	1-2*x-10*y*x, 1+2*y-5*x^2];
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  u = soln (j, xy);
  gu = solnp (j, xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  f = cast (f, class (xy));
endfunction

function C = boundcoeff(j, xy)
  if     j==1
    C = [0 -1];			# normal
  elseif j==2
    C = [0 1];			# normal
  elseif j==3
    C = [0 1];			# normal
  elseif j==4
    C = [-1 0];			# normal
  elseif j==5
    error ("Interface");
  elseif j==6
    C = [-1 0];			# normal
  elseif j==7
    C = [0 1];			# tangential
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs(j, xy)
  global Quads;
  quad = -1;
  for q = 1 : length (Quads)
    if any (abs(Quads{q}) == j)
      quad = q;
      break;
    endif
  endfor
  assert (quad > 0);
  u = soln (quad, xy);
  C = boundcoeff (j, xy);
  g = C * u;
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE2 (Geom, igrid, q, p, nrhs, \
		      @coeff1x, @coeff1y, @coeff0, @rhs, \
		      @boundcoeff, @boundrhs, @Jm, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  edge = zzs(i,3);
  quad = -1;
  for j = 1 : length (Quads)
    if any (abs(Quads{j}) == edge)
      quad = j;
      break;
    endif
  endfor
  assert (quad > 0);
  us2(:,i) = Jm (quad, edge, zzs(i,1:2)) * soln (quad, zzs(i,1:2));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
