## Test 10 of msnPDE.m
##
## Div-curl on 2x3 rectangles with double-slit
## and solution (z^2-1)^(3/2)
##
##
## $Id$
##

######################################
## 2x3 rectangles with double slit. ##
######################################
##
##
##       <--E8--><---E9--><--E10-->
##
##    ^  9-------10------11-------12^  	<---L3---
##    |  |     ^ |       | ^   	  | |
##    |  |     | |       | |   	  | |
##   E12 |   E14 |       | E16 	  | E18
##    |  |     | |   	 | |   	  | |
##    |  | Q6  v |    Q5 | v   Q4 | |
##    v  5-------6-------7--------8 v  	<---L2----
##    ^	 |<-E4-->|<--E6->|<--E7-->| ^
##    |	 |     	||   E5	 ||   	  | |
##   E11 |    E13|     	 |E15  	  | E17
##    |	 |     	||     	 ||    	  | |
##    |	 | Q1  	v| Q2  	 |v    Q3 | |
##    v	 1-------2-------3--------4 v  	<---L1-----
##       <--E1--><--E2---><--E3--->
##
##     	 ^     	 ^     	 ^     	  ^
## 	 |   	 |	 |	  |
## 	 L4    	 L5    	 L6    	  L7
## 	 |	 |     	 |     	  |
##

#
Pts = [-2 -1;			# 1
       -1 -1;			# 2
        1 -1;			# 3
        2 -1;			# 4
       -2  0;			# 5
       -1  0;			# 6
        1  0;			# 7
        2  0;			# 8
       -2  1;			# 9
       -1  1;			# 10
        1  1;			# 11
        2  1];			# 12
#
Lines = {[1 2 3 4];		# 1
	 [5 6 7 8];		# 2
	 [9 10 11 12];		# 3
	 [1 5 9];		# 4
	 [2 6 10];		# 5
	 [3 7 11];		# 6
	 [4 8 12]};		# 7
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 1 3 4;			# 3
	 2 5 6;			# 4
	 2 6 7;			# 5
	 2 6 7;			# 6
	 2 7 8;			# 7
	 3 9 10;		# 8
	 3 10 11;		# 9
	 3 11 12;		# 10
	 4 1 5;			# 11
	 4 5 9;			# 12
	 5 2 6;			# 13
	 5 6 10;		# 14
	 6 3 7;			# 15
	 6 7 11;		# 16
	 7 4 8;			# 17
	 7 8 12];		# 18
#
BEdges = [1 2 3 5 6 8 9 10 11 12 17 18];
#
## Anti-clockwise ordering
IEdges = [4 13 15 7 16 14];
#
# Anti-clockwise edges; anti-clockwise ordering
Quads = {[1 13 -4 -11];		# 1
	 [2 15 -5 -13];		# 2
	 [3 17 -7 -15];		# 3
	 [7 18 -10 -16];	# 4
	 [6 16 -9 -14];		# 5
	 [4 14 -8 -12]};	# 6
#
## Create grids on interior edges
ne = 13;
igrid = cell (6, 1);
for i = 1 : 6
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne));
endfor
#
nse = ne * ones(18,1);		# points on edges
##
ss = 10.0 * ones(6,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = ones (6,1) * [nx ny];	# No. of points for each Quad
#
M1 = 2.5;			# MSN amp factor
##
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

####################################################
############ Div-curl with (z^2-1)^(3/2) ###########
####################################################

p = 2;
q = 2;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = eye (2, class (x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 1; -1 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = zeros (2, 2, class (x));
endfunction

## (z^2-1)^(3/2) computed with correct branch cut
function u = foo (xy)
  i = sqrt(-1);
  z = xy(1) + xy(2)*i;
  u = (z^2-1)*sqrt(z-1)*sqrt(z+1);
endfunction

## (d/dz) (z^2-1)^(3/2) with correct branch cut
function up = foop (xy)
  i = sqrt(-1);
  z = xy(1) + xy(2)*i;
  up = 3 * z * sqrt(z-1) * sqrt(z+1);
endfunction

function u = soln (xy)
  u = foo (xy);
  u = [imag(u); real(u)];
  u = cast (u, class (xy));
endfunction

# [uI_x uI_y; uR_x uR_y]
function up = solnp (xy)
  up = foop(xy);
  up = [imag(up) real(up); real(up) -imag(up)];
  up = cast (up, class (xy));
endfunction

function du = curl (gu)
  du = gu(2,1) - gu(1,2);
endfunction

function du = div (gu)
  du = gu(1,1) + gu(2,2);
endfunction

## div and curl are both 0 in interior
function f = rhs(j, x)
  f = [0; 0];
  f = cast (f, class (x));
endfunction

## We want double-value on slit.
## uR = 0 on slit. uI = +- sqrt(-1) sqrt(1-x^2) on [-1,1] x 0.
## C = [+-1 0] on slit == tangential
## C = normal on outer boundary
function C = boundcoeff (j, xy)
  if     j==1
    C = [0 -1];			# normal
  elseif j==2
    C = [0 -1];			# normal
  elseif j==3
    C = [0 -1];			# normal
  elseif j==4
    error ("Interface");
  elseif j==5
    C = [1 0];			# tangential
  elseif j==6
    C = [1 0];			# tangential
  elseif j==7
    error ("Interface");
  elseif j==8
    C = [0 1];			# normal
  elseif j==9
    C = [0 1];			# normal
  elseif j==10
    C = [0 1];			# normal
  elseif j==11
    C = [-1 0];			# normal
  elseif j==12
    C = [-1 0];			# normal
  elseif j==13
    error ("Interface");
  elseif j==14
    error ("Interface");
  elseif j==15
    error ("Interface");
  elseif j==16
    error ("Interface");
  elseif j==17
    C = [1 0];			# normal
  elseif j==18
    C = [1 0];			# normal
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs (j, xy)
  x = xy(1);
  y = xy(2);
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
  if j==5
    g = [-(x^2-1)*sqrt(1-x^2)];	# uI- = -(x^2-1)*sqrt(1-x^2)
  elseif j==6
    g = [(x^2-1)*sqrt(1-x^2)];	# uI+ = +(x^2-1)*sqrt(1-x^2)
  endif
  g = cast (g, class(xy));
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
