## Test 12 of msnPDE.m
##
## 5x5 elastic equation in 2 rectangles size-by-side.
##
##
## $Id$
##

##################################
## Two rectangles side by side. ##
##################################
##
##    <--E3--><--E4--->
##    <-------L2------>	
## 	    	       	
## ^  4-------5-------6 ^
## |  |	    ^ |       |	|
## |  |	    | |	      |	|
## L3 |    L4 |       |	L5     	    
## |  |     | |	      |	|   
## |  |	    v |	      |	|
## v  1-------2-------3	v
##    		       
##    <------L1------->
##    <--E1--><--E2--->
##

#		
Pts = [0 0;			# 1
       1 0;			# 2
       2 0;			# 3
       0 1;			# 4
       1 1;			# 5
       2 1];			# 6
#		
Lines = {[1 2 3];		# 1
	 [4 5 6];		# 2
	 [1 4];			# 3
	 [2 5];			# 4
	 [3 6]};		# 5
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 2 4 5;			# 3
	 2 5 6;			# 4
	 3 1 4;			# 5
	 4 2 5;			# 6
	 5 3 6];		# 7
#
BEdges = [1 2 3 4 5 7];
#
IEdges = [6];
#
# Anti-clockwise
Quads = {[1 6 -3 -5];		# 1
	 [2 7 -4 -6]};		# 2
#
## Create a grid on E6
ne = 13;
igrid = {toEdge(Pts(2,:), Pts(5,:), cheb_zeros(ne))};
#
nse = ne * ones(7,1);		# points on edges
##
ss = 10.0 * ones(2,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = [nx ny;			# No. of points for each Quad
      nx ny];
#
M1 = 2.5;			# MSN amp factor
## 
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#######################################
############ Elastic  5x5 ###########
#######################################

p = 5;
q = 5;
nrhs = 1;

clear E;
clear nu;
global E = 1;			# Young's modulus
global nu = 0.25;		# Poisson's ration

function A1 = coeff1x (j, x)
  global E;
  global nu;
  f = E / ((1+nu) * (1-2*nu));
  D11 = f * (1 - nu);
  D12 = f * nu;
  D21 = f * nu;
  D22 = f * (1 - nu);
  D33 = f * (1 - 2 * nu) / 2;
  A1 = [0   0   1 0 0;
	0   0   0 0 1;
	D11 0   0 0 0;
	D12 0   0 0 0;
	0   D33 0 0 0];
  A1 = cast (A1, class (x));
endfunction

function A2 = coeff1y (j, x)
  global E;
  global nu;
  f = E / ((1+nu) * (1-2*nu));
  D11 = f * (1 - nu);
  D12 = f * nu;
  D21 = f * nu;
  D22 = f * (1 - nu);
  D33 = f * (1 - 2 * nu) / 2;
  A2 = [0   0 0 0 1;
	0   0 0 1 0;
	0 D12 0 0 0;
	0 D22 0 0 0;
	D33 0 0 0 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = [0 0  0  0  0;
       0 0  0  0  0;
       0 0 -1  0  0;
       0 0  0 -1  0;
       0 0  0  0 -1];
  B = cast (B, class (x));
endfunction

# [u1; u2; u1_x; u2_y; u2x+u1y]
function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       x^2 - 2*y^2 + x*y - x + 1;
       -2*x/(1+x^2+y^2)^2;
       -4*y+x;
       2*x+y-1-2*y/(1+x^2+y^2)^2];
  u = cast (u, class (xy));
endfunction

# [u1_x      u1_y;
#  u2_x      u2_y;
#  u1_xx     u1_xy;
#  u2_yx     u2_yy;
#  u1yx+u2xx u1yy+u2xy]
function up = solnp (xy)
  x = xy(1);
  y = xy(2);
  z = 1 + x^2 + y^2;
  up = [-2*x/z^2,          -2*y/z^2;
	2*x+y-1,           -4*y+x;
  	(8*x^2/z^3)-2/z^2, 8*x*y/z^3;
	1,                 -4;
	8*x*y/z^3+2,       (8*y^2/z^3)-2/z^2+1];
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  u = soln (xy);
  gu = solnp (xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  # f(3:5) = 0; # to be pure
  f = cast (f, class (xy));
endfunction

function C = boundcoeff (j, xy)
  displacement = [1 0 0 0 0;
		  0 1 0 0 0];	# displacement
  if     j==1
    C = [0 0 0 0 -1
	 0 0 0 -1 0];	# stress
  elseif j==2
    C = [0 0 0 0 -1
	 0 0 0 -1 0];	# stress
  elseif j==3
    C = [0 0 0 0 1
	 0 0 0 1 0];	# stress
  elseif j==4
    C = [0 0 0 0 1
	 0 0 0 1 0];	# stress
  elseif j==5
    C = displacement;		# displacement
  elseif j==6
    error ("Interface");
  elseif j==7
    n = rot90 (j);
    C = [0 0 1 0 0
	 0 0 0 0 1];	# stress
  else
    error("Unknown edge index")
  endif
  C = cast (C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

#
clear Pts;
clear Edges;
clear IEdges;

# Local Variables: #
# mode: octave     #
# End:             #
