## Test 14_10_2 of msnPDE.m
##
## 3x3 Poisson in polar coordinates on 3x1 rectangles on cylinder
##
## $Id$
##

###################################
## Three rectangles on cylinder. ##
###################################
##
##
##		    o-----------E3---------->
##	       A  A 5-----------------------6 A	 A    <--------L3
##	       |  | |	       	       	    | |	 |
##	       |  | |			    | |	 |
##	       | E6 |			    | E7 |
##	       |  | |			    | |	 |
##	       |  | |			    | |	 E9
##	      E8  | |  	       	       	    | |	 |
##	       |  o 3-----------------------4 o	 |    <--------L2
##	       |  A |o----------E2--------->| A	 |
##     	       |  | |			    | |	 |
##	       |  | |			    | |	 |
##	       | E4 |			    | E5 |
##	       |  | |			    | |	 |
##	       |  | |			    | |	 |
##	       o  o 1-----------------------2 o  o    <--------L1
## 	       	    o-----------E1---------->
##
## 		    ^  	       	       	    ^
## 		    |			    |
## 		    |			    |
## 		    L4 	       	       	    L5
##


#
Pts = [0 0;			# 1
       1 0;			# 2
       0 1;			# 3
       1 1;			# 4
       0 2;			# 5
       1 2];			# 6
#
Lines = {[1 2];		# 1
	 [3 4];		# 2
	 [5 6];		# 3
	 [1 3 5];	# 4
	 [2 4 5]};	# 5
#
Edges = [1 1 2;			# 1
	 2 3 4;			# 2
	 3 5 6;			# 3
	 4 1 3;			# 4
	 5 2 4;			# 5
	 4 3 5;			# 6
	 5 4 6;			# 7
	 4 1 5;			# 8
	 5 2 6];		# 9
#
BEdges = [4 5 6 7 8 9];
#
IEdges = [1 2 3];
#
# Anti-clockwise 1&2, clockwise 3
Quads = {[1 5 -2 -4];		# 1
	 [2 7 -3 -6];		# 2
	 [3 -9 -1 8]};		# 3
#
## Create grids on interior edges
ne = 20;
igrid = cell (3, 1);
for i = 1 : 3
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne));
endfor
#
nse = ne * ones(9,1);		# points on edges
##
ss = 10.0 * ones(3,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = ones(3,1) * [nx ny];	# No. of points for each Quad
#
M1 = 2.5;			# MSN amp factor
##
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#####################################################
########## Poisson polar coordinates 3x3 ############
#####################################################

p = 3;
q = 3;
nrhs = 1;

function A1 = coeff1x (j, xy)
  x = xy(1);
  y = xy(2);
  A1 = [x x^2 0;
	1 0   0;
	0 0   0];
  A1 = cast (A1, class(xy));
endfunction

function A2 = coeff1y (j, xy)
  A2 = [0 0 1;
	0 0 0;
	1 0 0];
  if j==3
    A2 = -A2;
  endif
  A2 = cast (A2, class (xy));
endfunction

function B = coeff0 (j, xy)
  B = [0  0  0;
       0 -1  0;
       0  0 -1];
  B = cast (B, class (xy));
endfunction

# [u; u_x; u_y]
function u = soln (ind, xy)
  x = xy(1);
  y = xy(2);
  # if ind==1
  #   y = 4-y;
  # endif
  u = [x^(5/2)*cos(5*pi*y/2);
       5*x^(3/2)*cos(5*pi*y/2)/2;
       -5*pi*x^(5/2)*sin(5*pi*y/2)/2];
  # if ind==1
  #   u(3) = -u(3);
  # endif
  u = cast (u, class (xy));
endfunction

# [u_x u_y; u_xx u_xy; u_xy u_yy]
function up = solnp (ind, xy)
  x = xy(1);
  y = xy(2);
  # if ind==1
  #   y = 4-y;
  # endif
  up = [5*x^(3/2)*cos(5*pi*y/2)/2,       -5*pi*x^(5/2)*sin(5*pi*y/2)/2;
	15*x^(1/2)*cos(5*pi*y/2)/4,     -25*pi*x^(3/2)*sin(5*pi*y/2)/4;
	-25*pi*x^(3/2)*sin(5*pi*y/2)/4, -25*pi^2*x^(5/2)*cos(5*pi*y/2)/4];
  # if ind==1
  #   up(3,1) = -up(3,1);
  #   up(1,2) = -up(1,2);
  #   up(2,2) = -up(2,2);
  # endif
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  if j==1 || j==2
    ind = 0;
  elseif j==3
    ind = 1;
  else
    error ("Unknown quad index")'
  endif
  u = soln (ind, xy);
  gu = solnp (ind, xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  # f(2:3) = 0; # to be pure
  f = cast (f, class (xy));
endfunction

function C = boundcoeff (j, xy)
  dirichlet = [1 0 0];
  if     j==1
    error ("Interface");
  elseif j==2
    error ("Interface");
  elseif j==3
    error ("Interface");
  elseif j==4
    C = dirichlet;
    # C = [0 -1 0];			# neumann
  elseif j==5
    C = dirichlet;
    # C = [0 1 0];			# neumann
  elseif j==6
    C = dirichlet;
    # C = [0 -1 0];		# neumann
  elseif j==7
    C = dirichlet;
    # C = [0 1 0];			# neumann
  elseif j==8
    C = dirichlet;
    # C = [0 -1 0];		# neumann
  elseif j==9
    C = dirichlet;
    # C = [0 1 0];		# neumann
  else
    error("Unknown edge index")
  endif
  C = cast (C, class(xy));
endfunction

function g = boundrhs(j, xy)
  if j==4 || j==5 || j==6 || j==7
    ind = 0;
  elseif j==8 || j==9
    ind = 1;
  elseif j==1 || j==2 || j==3
    error ("Interface edge index")
  else
    error ("Unknown edge index")'
  endif
  u = soln (ind, xy);
  C = boundcoeff (j, xy);
  g = C * u;
  g = cast (g, class(xy));
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (0, zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
