## Test 1_5 of msnPDE.m
##
## Div-curl on three quadrilaterals in T configuration
##
## $Id$
##

#####################################
## Three quads in T configuration. ##
#####################################
##
## 	  o-------E5-------->
##     	  o-------L3-------->
##  ^  ^  7-----------------8	^ ^
##  |  |  |	    	    |	| |
##  |  |  |	    	    |	| |
## E9  |  |o--E3-->o--E4--->|  	| E10
##  |  |  |o-------L2------>|	| |
##  o L4  |    	       	    |  L6 o
##  ^  |  4--------5--------6   | ^
##  |  |  |      ^ | ^      |	| |
##  |  |  |      | | |      |	| |
## E6  |  |     L5 | E7     |  	| E8
##  |  |  |      | | |      |	| |
##  |  |  |      o | o      |	| |
##  o  o  1--------2--------3	o o
##
##        o--------L1------->
##        o--E1---->o--E2--->
##
## The diagram is misleading as all three quads
## are *not* rectangular. However, all co-linearity
## indications are correct.

#
Pts = [0 0;			# 1
       4 -0.5;			# 2
       8 -1;			# 3
       0 2;			# 4
       3 2;			# 5
       6 2;			# 6
       0 4;			# 7
       4 5];			# 8
#
Lines = {[1 2 3];		# 1
	 [4 5 6];		# 2
	 [7 8];			# 3
	 [1 4 7];		# 4
	 [2 5]; 		# 5
	 [3 6 8]};		# 6
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 2 4 5;			# 3
	 2 5 6;			# 4
	 3 7 8;			# 5
	 4 1 4;			# 6
	 5 2 5;	        	# 7
	 6 3 6;			# 8
	 4 4 7;			# 9
	 6 6 8];		# 10
#
BEdges = [1 2 5 6 8 9 10];
#
IEdges = [3 4 7];
#
# Anti-clockwise edges
Quads = {[1 7 -3 -6];		# 1
	 [2 8 -4 -7];		# 2
	 [3 4 10 -5 -9]};	# 3
#
## Create a grid on E6
ne = 13;
igrid = cell (3, 1);
for i = 1 : 3
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne));
endfor	 
#
nse = ne * ones(10,1);		# points on edges
##
ss = 10.0 * ones(3,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = ones (3, 1) * [nx ny];	# No. of points for each Quad
#
M1 = 2.5;			# MSN amp factor
##
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## DIV-CURL #####################
#############################################

p = 2;
q = 2;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = eye (2, class (x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 1; -1 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = zeros (2, 2, class (x));
endfunction

function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       x^2 - 2*y^2 + x*y - x + 1];
  # u = [1+x-y+x*y+x^3; \
  #      x+y-x^2+y^2-5*y*x^2];
  u = cast (u, class (xy));
endfunction

# [u1_x u1_y; u2_x u2_y]
function up = solnp (xy)
  x = xy(1);
  y = xy(2);
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	2*x+y-1, -4*y+x];
  # up = [1+y+3*x^2, -1+x; \
  # 	1-2*x-10*y*x, 1+2*y-5*x^2];
  up = cast (up, class (xy));
endfunction

function du = curl (gu)
  du = gu(2,1) - gu(1,2);
endfunction

function du = div (gu)
  du = gu(1,1) + gu(2,2);
endfunction

function f = rhs(j, x)
  gu = solnp (x);
  f = [(div (gu));
       (curl (gu))];
  f = cast (f, class (x));
endfunction

function C = boundcoeff(j, xy)
  if     j==1
    C = [0 -1];			# normal
  elseif j==2
    C = [0 -1];			# normal
  elseif j==3
    error ("Interface");
  elseif j==4
    error ("Interface")
  elseif j==5
    C = [-1 0];			# tangential
  elseif j==6
    C = [-1 0];			# normal
  elseif j==7
    error ("Interface");
  elseif j==8
    C = [1 0];			# normal
  elseif j==9
    C = [-1 0];			# normal
  elseif j==10
    C = [1 0];			# normal
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
