## Test 1_6 of msnPDE.m
##
## Div-curl on triangle
##
## $Id$
##

##############
## Triangle ##
##############
##
##
##	    A  A /3\ A A
##	   /  /	/   \ \	\
##	  E5 / /     \ \ E4
##     	 /  / /	      \	\ \
##     	/ L3 /	       \ L2\
##     o  / /		\ \ o
##    A	 / 6_ L6    L5 _ 5 \ A
##   / 	/ /  \-	_   _ /	  \ \ \
##  /  / /  E9    7/  E8   \ \ \
## E6 /	/   	  |	    \ \	E3
##/  / /       	E7|L4	     \ \ \
##  / /	    	  |	      \	\ \
## o 1------------4------------2 o o
##   o-----------L1------------>
##   o----E1------>o-----E2---->
##
##
##

#
vt1 = [0 0];
vt2 = [8 0];
vt3 = [4 5];
clear Pts;
global \
Pts = zeros (7, 2);
Pts(1,:) = vt1;
Pts(2,:) = vt2;
Pts(3,:) = vt3;
Pts(4,:) = (vt1+vt2)/2;
Pts(5,:) = (vt2+vt3)/2;
Pts(6,:) = (vt3+vt1)/2;
Pts(7,:) = (vt1+vt2+vt3)/3;
#
Lines = {[1 4 2];		# 1
	 [2 5 3];		# 2
	 [1 6 3];		# 3
	 [4 7]; 		# 4
	 [7 5]; 		# 5
	 [7 6]};		# 6
#
clear Edges;
global \
Edges = [1 1 4;			# 1
	 1 4 2;			# 2
	 2 2 5;			# 3
	 2 5 3;			# 4
	 3 6 3;			# 5
	 3 1 6;			# 6
	 4 4 7;	        	# 7
	 5 7 5;			# 8
	 6 7 6];		# 9
#
BEdges = [1 2 3 4 5 6];
#
IEdges = [7 8 9];
#
# Anti-clockwise edges
Quads = {[1 7 9 -6];		# 1
	 [2 3 -8 -7];		# 2
	 [8 4 -5 -9]};  	# 3
#
## Create a grid on E6
ne = 13;
igrid = cell (3, 1);
for i = 1 : 3
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne));
endfor
#
nse = ne * ones(9,1);		# points on edges
##
ss = 10.0 * ones(3,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = ones (3, 1) * [nx ny];	# No. of points for each Quad
#
M1 = 2.5;			# MSN amp factor
##
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## DIV-CURL #####################
#############################################

p = 2;
q = 2;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = eye (2, class (x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 1; -1 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = zeros (2, 2, class (x));
endfunction

function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       x^2 - 2*y^2 + x*y - x + 1];
  # u = [1+x-y+x*y+x^3; \
  #      x+y-x^2+y^2-5*y*x^2];
  u = cast (u, class (xy));
endfunction

# [u1_x u1_y; u2_x u2_y]
function up = solnp (xy)
  x = xy(1);
  y = xy(2);
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	2*x+y-1, -4*y+x];
  # up = [1+y+3*x^2, -1+x; \
  # 	1-2*x-10*y*x, 1+2*y-5*x^2];
  up = cast (up, class (xy));
endfunction

function du = curl (gu)
  du = gu(2,1) - gu(1,2);
endfunction

function du = div (gu)
  du = gu(1,1) + gu(2,2);
endfunction

function f = rhs(j, x)
  gu = solnp (x);
  f = [(div (gu));
       (curl (gu))];
  f = cast (f, class (x));
endfunction

function tngnt = tangent (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  tngnt = vt2 - vt1;
endfunction

## Rotate by 90
function nrm = rot90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [t(2), -t(1)];
endfunction

## Rotate by -90
function nrm = rot_90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [-t(2), t(1)];
endfunction

function C = boundcoeff (j, xy)
  if     j==1
    C = [0 -1];			# normal
  elseif j==2
    C = [0 -1];			# normal
  elseif j==3
    C = rot90 (j);		# normal
  elseif j==4
    C = rot90 (j);		# normal
  elseif j==5
    C = rot_90 (j);		# normal
  elseif j==6
    C = rot_90 (j);		# normal
  elseif j==7
    error ("Interface");
  elseif j==8
    error ("Interface");
  elseif j==9
    error ("Interface");
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

#
clear Pts;
clear Edges;

# Local Variables: #
# mode: octave     #
# End:             #
