## Test 1_8 of msnPDE.m
##
## Div-curl on triangle with unaligned triangular hole
##
## $Id$
##

################################################
## Triangular hole unaligned inside triangle. ##
################################################
##
##
## See ./geom_8.gbb (Geogebra file).
##
##
##

#
l = 2/3;
#
vtA = [3 5];
vtB = [0 0];
vtC = [6 0];
vtD = l * vtA + (1-l) * vtB;
vtE = l * vtB + (1-l) * vtC;
vtF = l * vtC + (1-l) * vtA;
vtG = intersect ([vtD; vtC], [vtE; vtA]);
vtH = intersect ([vtF; vtB], [vtE; vtA]);
vtI = intersect ([vtF; vtB], [vtD; vtC]);
vtJ = intersect ([vtG; vtG + vtI - vtH], [vtC; vtA]);
vtK = intersect ([vtI; vtI + vtH - vtG], [vtB; vtC]);
vtL = intersect ([vtH; vtH + vtG - vtI], [vtA; vtB]);
#
global Pts;
Pts = [vtA;			# 1
       vtB;			# 2
       vtC;			# 3
       vtD;			# 4
       vtE;			# 5
       vtF;			# 6
       vtG;			# 7
       vtH;			# 8
       vtI;			# 9
       vtJ;			# 10
       vtK;			# 11
       vtL];			# 12
#
_A = 1;
_B = 2;
_C = 3;
_D = 4;
_E = 5;
_F = 6;
_G = 7;
_H = 8;
_I = 9;
_J = 10;
_K = 11;
_L = 12;
#
Lines = {[_A _D _L _B];		# 1
	 [_B _E _K _C];		# 2
	 [_C _F _J _A];		# 3
	 [_A _G _H _E];		# 4
	 [_B _H _I _F];		# 5
	 [_C _I _G _D];		# 6
	 [_I _K];		# 7
	 [_G _J];		# 8
	 [_H _L]};		# 9
#
global Edges;
Edges = [1 _A _D;		# 1
	 1 _D _L;		# 2
	 1 _L _B;		# 3
	 2 _B _E;		# 4
	 2 _E _K;		# 5
	 2 _K _C;		# 6
	 3 _C _F;		# 7
	 3 _F _J;		# 8
	 3 _J _A;		# 9
	 4 _G _H;		# 10
	 4 _H _E;		# 11
	 5 _H _I;		# 12
	 5 _I _F;		# 13
	 6 _I _G;		# 14
	 6 _G _D;		# 15
	 7 _I _K;		# 16
	 8 _G _J;		# 17
	 9 _H _L];		# 18
#
BEdges = [1 2 3 4 5 6 7 8 9 10 12 14];
#
IEdges = [11 16 13 17 15 18];
#
# Anti-clockwise edges
Quads = {[4 -11 18 3];		# 1
	 [5 -16 -12 11];	# 2
	 [6 7 -13 16];  	# 3
	 [13 8 -17 -14];	# 4
	 [17 9 1 -15];		# 5
	 [-18 -10 15 2]};			# 6
#
## create a grid on E6
ne = 13;
igrid = cell (6, 1);
for i = 1 : 6
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne));
endfor
#
nse = ne * ones(18,1);		# points on edges
##
ss = 10.0 * ones(6,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = ones (6, 1) * [nx ny];	# No. of points for each Quad
#
M1 = 2.5;			# MSN amp factor
##
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## DIV-CURL #####################
#############################################

p = 2;
q = 2;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = eye (2, class (x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 1; -1 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = zeros (2, 2, class (x));
endfunction

function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       x^2 - 2*y^2 + x*y - x + 1];
  # u = [1+x-y+x*y+x^3; \
  #      x+y-x^2+y^2-5*y*x^2];
  u = cast (u, class (xy));
endfunction

# [u1_x u1_y; u2_x u2_y]
function up = solnp (xy)
  x = xy(1);
  y = xy(2);
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	2*x+y-1, -4*y+x];
  # up = [1+y+3*x^2, -1+x; \
  # 	1-2*x-10*y*x, 1+2*y-5*x^2];
  up = cast (up, class (xy));
endfunction

function du = curl (gu)
  du = gu(2,1) - gu(1,2);
endfunction

function du = div (gu)
  du = gu(1,1) + gu(2,2);
endfunction

function f = rhs(j, x)
  gu = solnp (x);
  f = [(div (gu));
       (curl (gu))];
  f = cast (f, class (x));
endfunction

function tngnt = tangent (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  tngnt = vt2 - vt1;
endfunction

## Rotate by 90
function nrm = rot90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [t(2), -t(1)];
endfunction

## Rotate by -90
function nrm = rot_90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [-t(2), t(1)];
endfunction

function C = boundcoeff (j, xy)
  if     j==1
    C = rot90 (j);		# normal
  elseif j==2
    C = rot90 (j);		# normal
  elseif j==3
    C = rot90 (j);		# normal
  elseif j==4
    C = rot90 (j);		# normal
  elseif j==5
    C = rot90 (j);		# normal
  elseif j==6
    C = rot90 (j);		# normal
  elseif j==7
    C = rot90 (j);		# normal
  elseif j==8
    C = rot90 (j); 		# normal
  elseif j==9
    C = rot90 (j);		# normal
  elseif j==10
    C = tangent (j);		# tangential
  elseif j==11
    error ("Interface");
  elseif j==12
    C = tangent (j);		# tangential
  elseif j==13
    error ("Interface");
  elseif j==14
    C = tangent (14);		# tangential
  elseif j==15
    error ("Interface");
  elseif j==16
    error ("Interface");
  elseif j==17
    error ("Interface");
  elseif j==18
    error ("Interface");
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

#
clear Pts;
clear Edges;

# Local Variables: #
# mode: octave     #
# End:             #
