## Test 3_2 of msnPDE.m
## 
## 3x3 Poisson
##
## $Id$
##

####################################
## Three rectangles side by side. ##
####################################
##
##    <--E4--><--E5---><--E6--->
##    <-----------L2----------->
## 	    	       		
## ^  5-------6-------7--------8  ^
## |  |	    ^ |      ^|	       |  |
## |  |	    | |	     ||	       |  |
## L3 |    L4 |     L5|	       |  L6
## E7 |     | |	     ||	       |  E10
## |  |	    v |	     v|	       |  |  
## v  1-------2-------3--------4  v
##    		       	      	
##    <-----------L1----------->
##    <--E1--><--E2---><-- E3-->
##			      
			      
#			      
Pts = [0 0;		      	# 1
       1 0;		      	# 2
       2 0;		      	# 3
       3 0;    	       	      	# 4
       0 1;			# 5
       1 1;			# 6
       2 1;			# 7
       3 1];			# 8
#		
Lines = {[1 2 3 4];		# 1
	 [5 6 7 8];		# 2
	 [1 5];			# 3
	 [2 6];			# 4
	 [3 7]; 		# 5
	 [4 8]};		# 6
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 1 3 4;			# 3
	 2 5 6;			# 4
	 2 6 7;			# 5
	 2 7 8;			# 6
	 3 1 5; 	  	# 7
	 4 2 6;			# 8
	 5 3 7;			# 9
	 6 4 8];		# 10
#
BEdges = [1 2 3 4 5 6 7 10];
#
IEdges = [8 9];
#
# Anti-clockwise
Quads = {[1  8 -4 -7];		# 1
	 [2  9 -5 -8];		# 2
	 [3 10 -6 -9]};			# 3
#
## Create a grid on E8, E9
ne = 13;
igrid = {(toEdge (Pts(2,:), Pts(6,:), cheb_zeros (ne))), \
	 (toEdge (Pts(3,:), Pts(7,:), cheb_zeros (ne)))};
#
nse = ne * ones(10,1);		# points on edges
##
ss = 10.0 * ones(3,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = [nx ny;			# No. of points for each Quad
      nx ny;
      nx ny];
#
M1 = 2.5;			# MSN amp factor
## 
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## Poisson 4x3 ##################
#############################################

p = 3;
q = 3;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = [0 1 0;
	1 0 0;
	0 0 0];
  A1 = cast (A1, class(x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 0 1;
	0 0 0;
	1 0 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = [0  0  0;
       0 -1  0;
       0  0 -1];
  B = cast (B, class (x));
endfunction

# [u; u_x; u_y]
function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       -2*x/(1+x^2+y^2)^2;
       -2*y/(1+x^2+y^2)^2];
  u = cast (u, class (xy));
endfunction

# [u_x u_y; u_xx u_xy; u_xy u_yy]
function up = solnp (xy)
  x = xy(1);
  y = xy(2); 
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	(8*x^2/(1+x^2+y^2)^3)-2/(1+x^2+y^2)^2, 8*x*y/(1+x^2+y^2)^3;
	8*x*y/(1+x^2+y^2)^3, (8*y^2/(1+x^2+y^2)^3)-2/(1+x^2+y^2)^2];
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  u = soln (xy);
  gu = solnp (xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  # f(2:3) = 0; # to be pure
  f = cast (f, class (xy));
endfunction

function C = boundcoeff (j, xy)
  dirichlet = [1 0 0];
  if     j==1
    C = dirichlet;
    # C = [0 0 -1];			# neumann
  elseif j==2
    C = dirichlet;
    # C = [0 0 -1];			# neumann
  elseif j==3
    C = dirichlet;
    # C = [0 0 1];			# neumann
  elseif j==4
    C = dirichlet;
    # C = [0 0 1];			# neumann
  elseif j==5
    C = dirichlet;
    # C = [0 0 1];			# neumann
  elseif j==6
    C = dirichlet;
    # C = [0 0 1];   		        # neumann
  elseif j==7
    C = dirichlet;
    # C = [0 -1 0];		        # neumann
  elseif j==8
    error ("Interface");
  elseif j==9
    error ("Interface");
  elseif j==10
    C = dirichlet;
    # C = [0 1 0];			# neumann
  else
    error("Unknown edge index")
  endif
  C = cast (C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
  g = cast (g, class(xy));
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs);
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
