## Test 3_4 msnPDE.m
##
## 3x3 Poisson on diamond in rectangle
##
## $Id$
##

###########################
## Diamond in rectangle. ##
###########################
##
##
## 		  \ 			      /
## 		   \			     /
## 		    L11			    L12
## 		     \ 			   /
## 		      \	       	       	  /
##     	       	       U		 U
##
##	  <--------E9----------><-------E10-------->
##
##     ^  14-------------------15------------------16^ <------L5----
##     |  |    	       	     ^ |       	       	   | |
##     |  |	      	     | |		   | |
##    E18 |	 Q6   	    E19|	 Q5	   | E20
##     |  |	      	     | |	   	   | |
##     |  |    	       	     | |	   	   | |
##     v  |	      	     v |       	       	   | v
##     ^  11------------------12-------------------13^ <-------L4----
##     |  |<-------E7------A->/\<-A------E8------->| |
##     |  |    	       	  /  / 	\  \   	       	   | |
##    E16 |	      	 /  /	 \  \	   	   | E17
##     |  |   Q7       E23 /   	  \  E24    Q4 	   | |
##     |  |            /  /        \  \	       	   | |
##     v  |           U	 /          \  U       	   | v
##     ^  7-------------8           9--------------10^ <-------L3-----
##     |  |<-----E5---A->\         /<-A---E6------>| |
##    E14 |            \  \       /  / 	       	   | E15
##     |  |    Q8      E21 \     /  E22	   Q3	   | |
##     v  |              \  \   /  /           	   | v
##     ^  4---------------U---5---U----------------6 ^ <--------L2----
##     |  |<-------E3------^->|<---------E4------->| |
##     |  |                |  |        	   	   | |
##     |  |                |  |        	       	   | |
##     |  |                |  |       	   	   | |
##    E11 |      Q1       E12 |        	 Q2    	   | E13
##     |  |                |  |        	       	   | |
##     |  |    	       	   v  |	       	    	   | |
##     v  1-------------------2--------------------3 v <-------L1----
##
##	  <--------E1---------><---------E2-------->
##
##     	  ^    	     A 	      ^	   A   	       	   ^
## 	  |	    / 	      |	    \	   	   |
## 	  |	   / 	      |	     \	   	   |
## 	  L6	  L10	      L7      L9   	   L8
## 	  | 	 /   	      |	       \   	   |
## 	  |    	/      	      |		\  	   |

#
global \
Pts = [0   0;			# 1
       1   0;  	       	       	# 2
       2   0;			# 3
       0   1;			# 4
       1   1;			# 5
       2   1;			# 6
       0   2;			# 7
       0.5 2;			# 8
       1.5 2;			# 9
       2   2;			# 10
       0   3;			# 11
       1   3;			# 12
       2   3;			# 13
       0   4;			# 14
       1   4;			# 15
       2   4];			# 16
       
#
Lines = {[1 2 3];		# 1
	 [4 5 6];		# 2
	 [7 8 9 10];		# 3
	 [11 12 13];		# 4
	 [14 15 16];		# 5
	 [1 4 7 11 14];		# 6
	 [2 5 12 15];   	# 7
	 [3 6 10 13 16];	# 8
	 [5 8];			# 9
	 [5 9];			# 10
	 [9 12];		# 11
	 [8 12]};		# 12
#
global \
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 2 4 5;			# 3
	 2 5 6;			# 4
	 3 7 8;			# 5
	 3 9 10;		# 6
	 4 11 12;		# 7
	 4 12 13;		# 8
	 5 14 15;		# 9
	 5 15 16;		# 10
	 6 1 4;			# 11
	 7 2 5;			# 12
	 8 3 6;			# 13
	 6 4 7;			# 14
	 8 6 10;		# 15
	 6 7 11;		# 16
	 8 10 13;		# 17
	 6 11 14;		# 18
	 7 12 15;		# 19
	 8 13 16;		# 20
	 9 5 8;			# 21
	 10 5 9;		# 22
	 12 8 12;		# 23
	 11 9 12];		# 24
#
BEdges = [1 2 13 15 17 20 10 9 18 16 14 11 21 23 24 22];
#
## Ordering determines sparsity pattern. Must match igrid.
IEdges = [3 12 4 6 8 19 7 5];
#
# Anti-clockwise edges. Ordering determines sparsity pattern.
Quads = {[1 12 -3 -11];		# 1
	 [2 13 -4 -12];		# 2
	 [4 15 -6 -22];		# 3
	 [6 17 -8 -24];		# 4
	 [8 20 -10 -19];	# 5
	 [7 19 -9 -18];		# 6
	 [5 23 -7 -16];		# 7
	 [3 21 -5 -14]};	# 8
#
## Create grids on IEdges (8 of them)
ne = 13;
igrid = cell (8, 1);
for i = 1 : 8
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne));
endfor
#
nse = ne * ones (24,1);		# points on edges
##
ss = 10.0 * ones (8,1);		# Sobolev parameter for each Quad
nx = ne;
ny = ne;
ns = ones (8, 1) * [nx ny];     # No. of points for each Quad
#
M1 = 2.5;			# MSN amp factor
##
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## Poisson x3 ##################
#############################################

p = 3;
q = 3;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = [0 1 0;
	1 0 0;
	0 0 0];
  A1 = cast (A1, class(x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 0 1;
	0 0 0;
	1 0 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = [0  0  0;
       0 -1  0;
       0  0 -1];
  B = cast (B, class (x));
endfunction

# [u; u_x; u_y]
function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);
       -2*x/(1+x^2+y^2)^2;
       -2*y/(1+x^2+y^2)^2];
  u = cast (u, class (xy));
endfunction

# [u_x u_y; u_xx u_xy; u_xy u_yy]
function up = solnp (xy)
  x = xy(1);
  y = xy(2);
  up = [-2*x/(1+x^2+y^2)^2, -2*y/(1+x^2+y^2)^2;
  	(8*x^2/(1+x^2+y^2)^3)-2/(1+x^2+y^2)^2, 8*x*y/(1+x^2+y^2)^3;
	8*x*y/(1+x^2+y^2)^3, (8*y^2/(1+x^2+y^2)^3)-2/(1+x^2+y^2)^2];
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  u = soln (xy);
  gu = solnp (xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  # f(2:3) = 0; # to be pure
  f = cast (f, class (xy));
endfunction

## Rotate by 90
function nrm = rot90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [t(2), -t(1)];
endfunction

## Rotate by -90
function nrm = rot_90 (j)
  global Pts;
  global Edges;
  vt1 = Pts(Edges(j,2),:);
  vt2 = Pts(Edges(j,3),:);
  t = vt2 - vt1;
  nrm = [-t(2), t(1)];
endfunction

function C = boundcoeff (j, xy)
  dirichlet = [1 0 0];
  if     j==1
    C = dirichlet;
    # C = [0 0 -1];			# neumann
  elseif j==2
    C = dirichlet;
    # C = [0 0 -1];			# neumann
  elseif j==3
    error ("Interface");
  elseif j==4
    error ("Interface");
  elseif j==5
    error ("Interface");
  elseif j==6
    error ("Interface");
  elseif j==7
    error ("Interface");
  elseif j==8
    error ("Interface");
  elseif j==9
    C = dirichlet;
    # C = [0 0 1];			# neumann
  elseif j==10
    C = dirichlet;
    # C = [0 0 1];      		# neumann
  elseif j==11
    C = dirichlet;
    # C = [0 -1 0];			# neumann
  elseif j==12
    error ("Interface");
  elseif j==13
    C = dirichlet;
    # C = [0 1 0];			# neumann
  elseif j==14
    C = dirichlet;
    # C = [0 -1 0];			# neumann
  elseif j==15
    C = dirichlet;
    # C = [0 1 0];			# neumann
  elseif j==16
    C = dirichlet;
    # C = [0 -1 0];			# neumann
  elseif j==17
    C = dirichlet;
    # C = [0 1 0];			# neumann
  elseif j==18
    C = dirichlet;
    # C = [0 -1 0];			# neumann
  elseif j==19
    error ("Interface");
  elseif j==20
    C = dirichlet;
    # C = [0 1 0];			# neumann
  elseif j==21
    C = dirichlet;
    # C = [0, rot90(j)]  	# neumann
  elseif j==22
    C = dirichlet;
    # C = [0, rot_90(j)]         # neumann
  elseif j==23
    C = dirichlet;
    # C = [0, rot90(j)]  	# neumann
  elseif j==24
    C = dirichlet;
    # C = [0, rot_90(j)] 	# neumann
  else
    error("Unknown edge index")
  endif
  C = cast (C, class(xy));
endfunction

function g = boundrhs (j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
  g = cast (g, class(xy));
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs)
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)

# Local Variables: #
# mode: octave     #
# End:             #
