## Test 6 of msnPDE.m
## 
## Biharmonic equation 9x9 formulation
##
## $Id$
##

##################################
## Two rectangles side by side. ##
##################################
##
##    <--E3--><--E4--->
##    <-------L2------>	
## 	    	       	
## ^  4-------5-------6 ^
## |  |	    ^ |       |	|
## |  |	    | |	      |	|
## L3 |    L4 |       |	L5     	    
## |  |     | |	      |	|   
## |  |	    v |	      |	|
## v  1-------2-------3	v
##    		       
##    <------L1------->
##    <--E1--><--E2--->
##

#		
Pts = [0 0;			# 1
       1 0;			# 2
       2 0;			# 3
       0 1;			# 4
       1 1;			# 5
       2 1];			# 6
#		
Lines = {[1 2 3];		# 1
	 [4 5 6];		# 2
	 [1 4];			# 3
	 [2 5];			# 4
	 [3 6]};		# 5
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 2 4 5;			# 3
	 2 5 6;			# 4
	 3 1 4;			# 5
	 4 2 5;			# 6
	 5 3 6];		# 7
#
BEdges = [1 2 3 4 5 7];
#
IEdges = [6];
#
# Anti-clockwise
Quads = {[1 6 -3 -5];		# 1
	 [2 7 -4 -6]};		# 2
#
## Create a grid on E6
ne = 13;
igrid = {toEdge(Pts(2,:), Pts(5,:), cheb_zeros(ne))};
#
nse = ne * ones(7,1);		# points on edges
##
ss = 10.0 * ones(2,1);		# Sobolev for each Quad
nx = ne;
ny = ne;
ns = [nx ny;			# No. of points for each Quad
      nx ny];
#
M1 = 2.5;			# MSN amp factor
## 
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## Biharmonic eqn. 9x9 ##########
#############################################

p = 9;
q = 9;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = [0 0 0 0 0 0 1 0 0;
	1 0 0 0 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0;
	0 1 0 0 0 0 0 0 0;
	0 0 1 0 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0;
	0 0 0 1 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0];
  A1 = cast (A1, class(x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 0 0 0 0 0 0 2 1;
	0 0 0 0 0 0 0 0 0;
	1 0 0 0 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0;
	0 0 1 0 0 0 0 0 0;
	0 0 0 0 0 0 0 0 0;
	0 0 0 1 0 0 0 0 0;
	0 0 0 0 0 1 0 0 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = -[0 0 0 0 0 0 0 0 0;
	0 1 0 0 0 0 0 0 0;
	0 0 1 0 0 0 0 0 0;
	0 0 0 1 0 0 0 0 0;
	0 0 0 0 1 0 0 0 0;
	0 0 0 0 0 1 0 0 0;
	0 0 0 0 0 0 1 0 0;
	0 0 0 0 0 0 0 1 0;
	0 0 0 0 0 0 0 0 1];
  B = cast (B, class (x));
endfunction

# [u; u_x; u_y; u_xx; u_xy; u_yy; u_xxx; u_xxy; u_yyy]
function u = soln (xy)
  x = xy(1);
  y = xy(2);
  u = [1/(1 + x^2 + y^2);	                     # u
       -2*x/(1+x^2+y^2)^2;			     # u_x
       -2*y/(1+x^2+y^2)^2;			     # u_y
       (8*x^2/(1+x^2+y^2)^3) - 2/(1+x^2+y^2)^2;	     # u_xx
       8*x*y/(1+x^2+y^2)^3;			     # u_xy
       (8*y^2/(1+x^2+y^2)^3) - 2/(1+x^2+y^2)^2;	     # u_yy
       (24*x/(1+x^2+y^2)^3) - 48*x^3/(1+x^2+y^2)^4;  # u_xxx
       (8*y/(1+x^2+y^2)^3) - 48*x^2*y/(1+x^2+y^2)^4; # u_xxy
       (24*y/(1+x^2+y^2)^3) - 48*y^3/(1+x^2+y^2)^4]; # u_yyy
  u = cast (u, class (xy));
endfunction

## [u_x    u_y;
##  u_xx   u_xy;
##  u_yx   u_yy;
##  u_xxx  u_xxy;
##  u_xyx  u_xyy;
##  u_yyx  u_yyy;
##  u_xxxx u_xxxy;
##  u_xxyx u_xxyy;
##  u_yyyx u_yyyy];
function up = solnp (xy)
  x = xy(1);
  y = xy(2);
  z = 1 + x^2 + y^2;
  up = [-2*x/z^2, -2*y/z^2;                             # u_x    u_y
  	8*x^2/z^3 - 2/z^2, 8*x*y/z^3;			# u_xx   u_xy
	8*x*y/z^3, 8*y^2/z^3 - 2/z^2;			# u_yx   u_yy
	24*x/z^3 - 48*x^3/z^4, 8*y/z^3 - 48*x^2*y/z^4;	# u_xxx  u_xxy
	8*y/z^3 - 48*x^2*y/z^4, 8*x/z^3 - 48*x*y^2/z^4; # u_xyx  u_xyy
	8*x/z^3 - 48*x*y^2/z^4, 24*y/z^3 - 48*y^3/z^4;  # u_yyx  u_yyy
	24/z^3 - 288*x^2/z^4 + 384*x^4/z^5,	\	# u_xxxx
	384*x^3*y/z^5 - 144*x*y/z^4;			#        u_xxxy
	384*x^3*y/z^5 - 144*x*y/z^4,		\	# u_xxyx
	8/z^3 - 48*(x^2+y^2)/z^4 + 384*x^2*y^2/z^5;	#        u_xxyy
	384*x*y^3/z^5 - 144*x*y/z^4,		\	# u_yyyx
	24/z^3 - 288*y^2/z^4 + 384*y^4/z^5];		#        u_yyyy
  assert (rows (up), 9);
  assert (columns (up), 2);
  up = cast (up, class (xy));
endfunction

function f = rhs (j, xy)
  u = soln (xy);
  gu = solnp (xy);
  A1 = coeff1x (j, xy);
  A2 = coeff1y (j, xy);
  B = coeff0 (j, xy);
  f = A1 * gu(:,1) + A2 * gu(:,2) + B * u;
  # f(2:9) = 0; # to be pure
  f = cast (f, class (xy));
endfunction

function C = boundcoeff (j, xy)
  if     j==1
    C = [1 0  0 0 0 0 0 0 0;
	 0 0 -1 0 0 0 0 0 0];
  elseif j==2
    C = [1 0  0 0 0 0 0 0 0;
	 0 0 -1 0 0 0 0 0 0];
  elseif j==3
    C = [1 0 0 0 0 0 0 0 0;
	 0 0 1 0 0 0 0 0 0];
  elseif j==4
    C = [1 0 0 0 0 0 0 0 0;
	 0 0 1 0 0 0 0 0 0];
  elseif j==5
    C = [1  0 0 0 0 0 0 0 0;
	 0 -1 0 0 0 0 0 0 0];
  elseif j==6
    error ("Interface");
  elseif j==7
    C = [1 0 0 0 0 0 0 0 0;
	 0 1 0 0 0 0 0 0 0];
  else
    error("Unknown edge index")
  endif
  C = cast (C, class(xy));
endfunction

function g = boundrhs(j, xy)
  u = soln (xy);
  C = boundcoeff (j, xy);
  g = C * u;
  g = cast (g, class(xy));
endfunction

#############################################
#############################################
#############################################

[zzs, us1] = msnPDE (Geom, igrid, q, p, nrhs, \
		    @coeff1x, @coeff1y, @coeff0, @rhs, \
		    @boundcoeff, @boundrhs, ss, ns, nse, M1);
#

m = rows (zzs)
us1 = reshape (us1, p, m);

#

us2 = zeros (p, m);
for i = 1 : m
  us2(:,i) = soln (zzs(i,:));
endfor
us2;

#

err = norm (us1-us2, Inf) / norm (us2, Inf)
err1 = norm (us1(1,:)-us2(1,:), Inf) / norm (us2(1,:), Inf)

# Local Variables: #
# mode: octave     #
# End:             #
