## Self-consistency test 1 of msnPDE2.m
##
## $Id$
##

##################################
## Two rectangles side by side. ##
##################################
##
##    <--E3--><--E4--->
##    <-------L2------>	
## 	    	       	
## ^  4-------5-------6 ^
## |  |	    ^ |       |	|
## |  |	    | |	      |	|
## L3 |    L4 |       |	L5     	    
## |  |     | |	      |	|   
## |  |	    v |	      |	|
## v  1-------2-------3	v
##    		       
##    <------L1------->
##    <--E1--><--E2--->
##

#		
Pts = [0 0;			# 1
       1 0;			# 2
       2 0;			# 3
       0 1;			# 4
       1 1;			# 5
       2 1];			# 6
#		
Lines = {[1 2 3];		# 1
	 [4 5 6];		# 2
	 [1 4];			# 3
	 [2 5];			# 4
	 [3 6]};		# 5
#
Edges = [1 1 2;			# 1
	 1 2 3;			# 2
	 2 4 5;			# 3
	 2 5 6;			# 4
	 3 1 4;			# 5
	 4 2 5;			# 6
	 5 3 6];		# 7
#
BEdges = [1 2 3 4 5 7];
#
IEdges = [6];
#
# Anti-clockwise
Quads = {[1 6 -3 -5];		# 1
	 [2 7 -4 -6]};		# 2
## 
Geom.Pts = Pts;
Geom.Lines = Lines;
Geom.Edges = Edges;
Geom.BEdges = BEdges;
Geom.IEdges = IEdges;
Geom.Quads = Quads;

#############################################
############## DIV-CURL #####################
#############################################

p = 2;
q = 2;
nrhs = 1;

function A1 = coeff1x (j, x)
  A1 = eye (2, class (x));
endfunction

function A2 = coeff1y (j, x)
  A2 = [0 1; -1 0];
  A2 = cast (A2, class (x));
endfunction

function B = coeff0 (j, x)
  B = zeros (2, 2, class (x));
endfunction

function f = rhs(j, x)
  f = [0; 0];			# no charge density
  f = cast (f, class (x));
endfunction

function C = boundcoeff(j, xy)
  if     j==1
    C = [0 -1];			# normal
  elseif j==2
    C = [0 -1];			# normal
  elseif j==3
    C = [0 1];			# normal
  elseif j==4
    C = [0 1];			# normal
  elseif j==5
    C = [0 -1];			# tangential
  elseif j==6
    error("Interface");
  elseif j==7
    C = [1 0];			# normal
  else
    error("Unknown edge index")
  endif
  C = cast(C, class(xy));
endfunction

function g = boundrhs(j, xy)
  x = xy(1);
  y = xy(2);
  if     j==1
    g = [x];			# normal
  elseif j==2
    g = [x];			# normal
  elseif j==3
    g = [-x];			# normal
  elseif j==4
    g = [-x];			# normal
  elseif j==5
    g = [0];			# tangential
  elseif j==6
    error("Interface");
  elseif j==7
    g = [1];			# normal
  else
    error("Unknown edge index")
  endif
  g = cast(g, class(xy));
endfunction

function jm = Jm (p, j, xy)
  jm = eye(2, class(xy));	# single material
endfunction

#############################################
#############################################
#############################################

#
s = 10.0;
ss = s * ones (length(Quads),1); # Sobolev for each Quad
M1 = 2.5;			# MSN amp factor

#
ne1 = 13

## Create a grid on interfaces
igrid1 = cell(length(IEdges),1);
for i = 1 : length(IEdges)
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid1{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne1));
endfor

#
nse = ne1 * ones (length(Edges),1);  # points on edges
##
nx = ne1;
ny = ne1;
ns = ones (length(Quads),1) * [nx ny];	# No. of points for each Quad
#

## 
[zzs1, us1] = msnPDE2 (Geom, igrid1, q, p, nrhs, \
		      @coeff1x, @coeff1y, @coeff0, @rhs, \
		      @boundcoeff, @boundrhs, @Jm, ss, ns, nse, M1);
#

m1 = rows (zzs1);
us1 = reshape (us1, p, m1);

## Re-run test with larger ne

#
ne2 = ne1+1

## Create a grid on interfaces
igrid2 = cell(length(IEdges),1);
for i = 1 : length(IEdges)
  ei = IEdges(i);
  vt1 = Edges(ei, 2);
  vt2 = Edges(ei, 3);
  igrid2{i} = toEdge (Pts(vt1,:), Pts(vt2,:), cheb_zeros(ne2));
endfor

#
nse = ne2 * ones (length(Edges),1);  # points on edges
##
nx = ne2;
ny = ne2;
ns = ones (length(Quads),1) * [nx ny];	# No. of points for each Quad
#

## 
[zzs2, us2] = msnPDE2 (Geom, igrid2, q, p, nrhs, \
		      @coeff1x, @coeff1y, @coeff0, @rhs, \
		      @boundcoeff, @boundrhs, @Jm, ss, ns, nse, M1);
#

m2 = rows (zzs2);
us2 = reshape (us2, p, m2);

## Evaluate error between two "solutions"

err = zeros (p, 1);		# error per component
nrm = zeros (p, 1);		# norm per component
for i = 1 : p
  k1 = 1;			# row index for current edge in zzs1
  k2 = 1;			# row index for current edge in zzs2
  for j = 1 : length(IEdges)
    edge = IEdges(j);
    vt1 = Pts(Edges(edge, 2),:);
    vt2 = Pts(Edges(edge, 3),:);
    grid1 = igrid1{j};
    grid2 = igrid2{j};
    ks1 = length (grid1);
    ks2 = length (grid2);
    assert (all (zzs1(k1:k1+ks1-1,3) == edge));
    assert (all (zzs2(k2:k2+ks2-1,3) == edge));
    zs1 = zzs1(k1:k1+ks1-1,1:2);
    xs1 = fromEdge (vt1, vt2, zs1);
    zs2 = zzs2(k2:k2+ks2-1,1:2);
    xs2 = fromEdge (vt1, vt2, zs2);
    fs1 = us1(i,k1:k1+ks1-1);
    fs1 = fs1';
    cfs2 = msn_reinterp (s, M1, -1, 1, xs1, fs1, xs2);
    fs2 = us2(i,k2:k2+ks2-1);
    fs2 = fs2';
    err(i) = max (err(i), norm(fs2-cfs2,Inf));
    nrm(i) = max (nrm(i), norm(fs2,Inf));
    k1 = k1 + ks1;
    k2 = k2 + ks2;
  endfor
endfor

disp("error normInf");
[err nrm]

# Local Variables: #
# mode: octave     #
# End:             #
